using System.Collections.Concurrent;

namespace ArmaRAMDb
{
    internal class ListStore
    {
        private static readonly ConcurrentDictionary<string, List<string>> _lists = RAMDb._lists;

        public static bool ListPush(string listName, string value)
        {
            if (_lists.TryGetValue(listName, out var list))
            {
                lock (list)
                {
                    list.Add(value);
                    Main.Log($"ListPush: {listName} - Value: {value}", "debug");
                    return true;
                }
            }

            Main.Log($"List not found: {listName}", "debug");
            return false;
        }

        public static string ListGet(string listName, int index)
        {
            if (_lists.TryGetValue(listName, out var list))
            {
                lock (list)
                {
                    string value = index >= 0 && index < list.Count ? list[index] : string.Empty;
                    Main.Log($"ListGet: {listName} - Index: {index} - Value: {value}", "debug");
                    return value;
                }
            }

            Main.Log($"List not found: {listName}", "debug");
            return string.Empty;
        }

        public static List<string> GetList(string listName)
        {
            Main.Log($"Getting contents for list: {listName}", "debug");
            
            if (_lists.TryGetValue(listName, out var list))
            {
                var contents = new List<string>(list);
                Main.Log($"Found {contents.Count} entries in list {listName}", "debug");
                foreach (var item in contents)
                {
                    Main.Log($"Value: {item}", "debug");
                }
                return contents;
            }
            
            Main.Log($"List not found: {listName}", "debug");
            return [];
        }

        public static bool ListDelete(string listName, int index)
        {
            if (_lists.TryGetValue(listName, out var list))
            {
                lock (list)
                {
                    if (index >= 0 && index < list.Count)
                    {
                        string value = list[index];
                        list.RemoveAt(index);
                        Main.Log($"ListDelete: {listName} - Index: {index} - Value: {value}", "debug");
                        return true;
                    }
                }
            }

            Main.Log($"List not found: {listName}", "debug");
            return false;
        }

        public static bool DeleteList(string listName)
        {
            var result = _lists.TryRemove(listName, out _);

            Main.Log($"DeleteList: {listName} - Success: {result}", "debug");
            return result;
        }
    }
}